%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Bandit Comparison
%
% Estimate the probability of pulling the optimal lever and the regret
% as a function of the number of trials for different bandit algorithms.
% The rewards follow a Bernoulli distribution with unknown biases.
%
% (c) 2014 Pedro A. Ortega <pedro.ortega@gmail.com>
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Get ready.
close all;
clear all;
clc;

%% Script configuration.

T = 1000; % Number of timesteps.
N = 1000; % Number of Monte Carlo samples.

% algorithms
algorithms = {'gittinsRun', 'ucb1Run', 'thompsonRun'};
names  = {'Gittins', 'UCB1', 'Thompson Sampling'};
colors = {[1 0 0], [0 1 0], [0 0 1]};

% axes for plots
probAxis   = [0, N, 0.4, 1.0];
regretAxis = [0, N, 0, 30];

% Bandit parameters.
theta = [0.55, 0.45];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Initialize.
% Total number of algorithms.
nAlgs = length(algorithms);

% Sums of actions and rewards.
ActionSum = zeros(nAlgs, T);
RewardSum = zeros(nAlgs, T);

% Open figure.
h = figure();
set(h, 'Position', [100, 100, 1000, 500]);
    
for alg=1:nAlgs
    % Get handle of algorithm to run.
    fprintf('Running algorithm: %s\n', algorithms{alg});
    fhandle = str2func(algorithms{alg});

    %% Generate Monte Carlo simulations.
    for n=1:N
        % Generate one run.
        [Action, Reward, Prob] = fhandle(theta, T);

        % Collect stats.
        RewardSum(alg, :) = RewardSum(alg, :) + Reward;
        ActionSum(alg, :) = ActionSum(alg, :) + Action;
        
        % Report progress.
        if (mod(n,100) == 0)
            fprintf('Samples: %5d\n', n);
        end
    end
    
    %% Monte Carlo estimates.
    PAction = ActionSum(alg, :)/N;
    Regret  = max(theta)*[1:T] - cumsum(RewardSum(alg, :)/N);
    
    %% Plot.
    
    % Plot probability of pulling the best action.
    subplot(1,2,1);
    plot(PAction, 'Color', colors{alg});
    hold on;
    title('Probability of Optimal Action');
    xlabel('Trial');
    ylabel('Probability');
    axis square;
    axis(probAxis);
    
    % Plot regret.
    subplot(1,2,2);
    plot(Regret, 'Color', colors{alg});
    hold on;
    title('Regret');
    xlabel('Trial');
    ylabel('Probability');
    axis square;
    axis(regretAxis);
    
    drawnow;
end

%% Finish.
% Place legend.
subplot(1,2,1);
legend(names, 'Location', 'SouthEast');
subplot(1,2,2);
legend(names, 'Location', 'SouthEast');

% END OF SCRIPT